import { ROUTE_IDS } from './constants';
import type { ResetPasswordRequest } from './auth/local/resetPassword/types';
import type { Subset } from './utils/types';
import type { LocalSignupRequest } from './auth/local/signup/types';
import { PayinRouteParams } from './payin/route/types';
import { CreateBlogRequest } from './blog/parsers';
import { ChannelGetOneParams, ChannelListParams } from './channel/parsers';
import { LocalSigninRequest } from './auth/local/signin/types';
import {
  MixspaceCreateBody,
  MixspaceDeleteParams,
  MixspaceGetOneParams,
  MixspaceListParams,
} from './mixspace/api.types';
import { PayinMethodUpsertParams } from './payinMethod/api.types';

export type RouteId = typeof ROUTE_IDS[number];

interface ApiType<P, Q, B> {
  params?: P;
  query?: Q;
  body?: B;
}

interface BodyType<B> extends ApiType<unknown, unknown, B> {
  body: B;
}

interface ParamsType<P> extends ApiType<P, unknown, unknown> {
  params: P;
}

export interface ApiTypes {
  resetPassword: BodyType<ResetPasswordRequest>;
  localSignup: BodyType<LocalSignupRequest>;
  localSignin: BodyType<LocalSigninRequest>;
  signout: BodyType<ApiType<unknown, unknown, unknown>>;
  googleAuthInit: BodyType<ApiType<unknown, unknown, unknown>>;
  createBlog: BodyType<CreateBlogRequest>;
  fetchBlog: ParamsType<ChannelGetOneParams>;
  fetchBlogs: ParamsType<ChannelListParams>;
  payinRoute: ParamsType<PayinRouteParams>;
  createMixspace: ParamsType<MixspaceCreateBody>;
  updateMixspace: ParamsType<MixspaceCreateBody>;
  fetchMixspaces: ParamsType<MixspaceListParams>;
  fetchMixspace: ParamsType<MixspaceGetOneParams>;
  deleteMixspace: ParamsType<MixspaceDeleteParams>;
  upsertPayinMethod: ParamsType<PayinMethodUpsertParams>;
}

// eslint-disable-next-line
type AssertApiTypes1 = Subset<RouteId, keyof ApiTypes>;
// eslint-disable-next-line
type AssertApiTypes2 = Subset<keyof ApiTypes, RouteId>;
